require "rails_helper"

module Strain
  RSpec.describe Variant, type: :model do

    it "has a valid factory" do
      expect(build(:variant)).to be_valid
    end

    it "is invalid without a name" do
      expect(build(:variant, name: nil)).to be_invalid
    end

    it "is invalid without a Schema" do
      expect(build(:variant, schema: nil)).to be_invalid
      expect(build(:variant, schema: {})).to be_invalid
    end

    it "is invalid with an invalid JSON Schema" do
      expect(build(:variant, schema: {"type": "nonsense"})).to be_invalid
    end

    it "raises a database error with a nonunique name" do
      a = create(:variant)
      b = build(:variant, name: a.name)
      expect {b.save(validate: false)}.to raise_error(ActiveRecord::RecordNotUnique)
    end

    it "is invalid with a nonunique name" do
      a = create(:variant)
      b = build(:variant, name: a.name)
      expect(b).to be_invalid
    end

    it "is invalid unless root schema type is object" do
      a = build(:variant, schema: {"type": "string"})
      expect(a).to be_invalid
      expect(a.errors.messages[:schema]).to include("must have an object as root element.")
    end

  end
end
